# ファイル名
FNAME_MAPPING = '_mapping.yaml'

# レイアウト
LAYOUT_TABLE = 'table'
LAYOUT_MATRIX = 'matrix'
LAYOUT_SINGLE = 'single'

# カラム定義
BIND_TYPE_FIXED = 'fixed'
BIND_TYPE_SELECTABLE = 'selectable'
BIND_TYPE_CALL = 'call'
# カラム定義（特殊タイプ）
BIND_TYPE_REF_ID = 'ref_id'
BIND_TYPE_FOREIGN_KEY = "foreign_key"
BIND_TYPE_EMBEDDED_DATA = "embedded_data"

# callのタイプ
CALL_TYPE_UUID = 'uuid'

# FIXME セルタイプ
FIELD_TYPE_STRING = 'string'
FIELD_TYPE_SELECTABLE = 'singleSelect'
FIELD_TYPE_SYSTEM = 'system'

# 同期モード
SYNC_MODE_DROP_CREATE = 'drop_create'
